@@ -46,7 +46,7 @@
 #define PPP_DECNETCP	0x8027	/* DECnet Control Protocol */
 #define PPP_APPLECP	0x8029	/* Appletalk Control Protocol */
 #define PPP_IPXCP	0x802b	/* Novell IPX Control Protocol */
-#define PPP_STIICP	0x8033	/* Strean Protocol Control Protocol */
+#define PPP_STIICP	0x8033	/* Stream Protocol Control Protocol */
 #define PPP_VINESCP	0x8035	/* Banyan Vines Control Protocol */
 #define PPP_IPV6CP	0x8057	/* IPv6 Control Protocol */
 #define PPP_MPLSCP      0x8281  /* rfc 3022 */
