@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *
- * The Nmap Security Scanner is (C) 1996-2023 Nmap Software LLC ("The Nmap
+ * The Nmap Security Scanner is (C) 1996-2025 Nmap Software LLC ("The Nmap
  * Project"). Nmap is also a registered trademark of the Nmap Project.
  *
  * This program is distributed under the terms of the Nmap Public Source
@@ -40,15 +40,16 @@
  * right to know exactly what a program is going to do before they run it.
  * This also allows you to audit the software for security holes.
  *
- * Source code also allows you to port Nmap to new platforms, fix bugs, and add
- * new features. You are highly encouraged to submit your changes as a Github PR
- * or by email to the dev@nmap.org mailing list for possible incorporation into
- * the main distribution. Unless you specify otherwise, it is understood that
- * you are offering us very broad rights to use your submissions as described in
- * the Nmap Public Source License Contributor Agreement. This is important
- * because we fund the project by selling licenses with various terms, and also
- * because the inability to relicense code has caused devastating problems for
- * other Free Software projects (such as KDE and NASM).
+ * Source code also allows you to port Nmap to new platforms, fix bugs, and
+ * add new features. You are highly encouraged to submit your changes as a
+ * Github PR or by email to the dev@nmap.org mailing list for possible
+ * incorporation into the main distribution. Unless you specify otherwise, it
+ * is understood that you are offering us very broad rights to use your
+ * submissions as described in the Nmap Public Source License Contributor
+ * Agreement. This is important because we fund the project by selling licenses
+ * with various terms, and also because the inability to relicense code has
+ * caused devastating problems for other Free Software projects (such as KDE
+ * and NASM).
  *
  * The free version of Nmap is distributed in the hope that it will be
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,7 +59,7 @@
  *
  ***************************************************************************/
 
-/* $Id: osscan.h 38653 2023-04-14 17:11:46Z dmiller $ */
+/* $Id: osscan.h 39253 2025-07-15 18:24:40Z dmiller $ */
 
 #ifndef OSSCAN_H
 #define OSSCAN_H
@@ -244,7 +245,7 @@ struct ObservationPrint {
   FingerPrintScan scan_info;
   std::vector<FingerTest> extra_tests;
   const char *getInfo(FingerPrintScan::Attribute attr) const {
-    if (attr >= FingerPrintScan::MAX_ATTR)
+    if (attr >= FingerPrintScan::MAX_ATTR || attr < 0)
       return NULL;
     return scan_info.values[static_cast<int>(attr)];
   }
@@ -319,5 +320,7 @@ const char *mergeFPs(FingerPrint *FPs[], int numFPs, bool isGoodFP,
                            const u8 *mac, int openTcpPort, int closedTcpPort,
                            int closedUdpPort, bool wrapit);
 
+// Internal matching function. Defined here for test purposes.
+bool expr_match(const char *val, size_t vlen, const char *expr, size_t explen, bool do_nested=false);
 #endif /*OSSCAN_H*/
 
