@@ -60,7 +60,7 @@
  *
  ***************************************************************************/
 
-/* $Id: scan_engine.h 39083 2025-02-26 17:44:43Z dmiller $ */
+/* $Id: scan_engine.h 39232 2025-06-23 20:54:32Z dmiller $ */
 
 #ifndef SCAN_ENGINE_H
 #define SCAN_ENGINE_H
@@ -68,7 +68,7 @@
 #include "scan_lists.h"
 #include "probespec.h"
 
-#include <dnet.h>
+#include "libnetutil/netutil.h"
 
 #include "timing.h"
 
@@ -626,7 +626,7 @@ public:
   const struct scan_lists *ports;
   int rawsd; /* raw socket descriptor */
   pcap_t *pd;
-  eth_t *ethsd;
+  netutil_eth_t *ethsd;
   u32 seqmask; /* This mask value is used to encode values in sequence
                   numbers.  It is set randomly in UltraScanInfo::Init() */
   u16 base_port;
