@@ -60,7 +60,7 @@
  *
  ***************************************************************************/
 
-/* $Id: targets.cc 39125 2025-04-16 00:00:05Z dmiller $ */
+/* $Id: targets.cc 39245 2025-07-11 17:08:57Z dmiller $ */
 
 
 #include <nbase.h>
@@ -498,45 +498,51 @@ static void refresh_hostbatch(HostGroupState *hs, struct addrset *exclude_group,
     hoststructfry(hs->hostbatch, hs->current_batch_sz);
   }
 
-  /* First I'll do the ARP ping if all of the machines in the group are
-     directly connected over ethernet.  I may need the MAC addresses
-     later anyway. */
-  /* No other interface types are supported by ND ping except devt_ethernet
-     at the moment. */
-  if (hs->hostbatch[0]->ifType() == devt_ethernet &&
-      hs->hostbatch[0]->directlyConnected() &&
-      o.sendpref != PACKET_SEND_IP_STRONG &&
-      o.implicitARPPing) {
-    arpping(hs->hostbatch, hs->current_batch_sz);
-    arpping_done = true;
-  }
-
   gettimeofday(&now, NULL);
-  if ((o.sendpref & PACKET_SEND_ETH) &&
-      hs->hostbatch[0]->ifType() == devt_ethernet) {
-    for (i=0; i < hs->current_batch_sz; i++) {
-      if (!(hs->hostbatch[i]->flags & HOST_DOWN) &&
-          !hs->hostbatch[i]->timedOut(&now)) {
-        if (!setTargetNextHopMAC(hs->hostbatch[i])) {
-          error("%s: Failed to determine dst MAC address for target %s",
-              __func__, hs->hostbatch[i]->NameIP());
-          hs->hostbatch[i]->flags = HOST_DOWN;
-          hs->hostbatch[i]->reason.reason_id = ER_NOROUTE;
+  Target *current_target = hs->hostbatch[0];
+
+  /* If there's a chance we can do ARP ping or may need the MAC address,
+   * we'll do the extra check. Some things like VPN claim devt_ethernet
+   * but are not DLT_EN10MB. */
+  if (current_target->ifType() == devt_ethernet &&
+        o.sendpref != PACKET_SEND_IP_STRONG) {
+    netutil_eth_t *eth = eth_open_cached(current_target->deviceName());
+    if (DLT_EN10MB == netutil_eth_datalink(eth)) {
+      // Do ARP/ND if possible
+      if (current_target->directlyConnected() &&
+          o.implicitARPPing) {
+        arpping(hs->hostbatch, hs->current_batch_sz);
+        arpping_done = true;
+      }
+      // If we want to do layer-2 sending, we'll need a MAC address.
+      if ((o.sendpref & PACKET_SEND_ETH)) {
+        for (i=0; i < hs->current_batch_sz; i++) {
+          current_target = hs->hostbatch[i];
+          if (!(current_target->flags & HOST_DOWN) &&
+              !current_target->timedOut(&now)) {
+            if (!setTargetNextHopMAC(current_target)) {
+              error("%s: Failed to determine dst MAC address for target %s",
+                  __func__, current_target->NameIP());
+              current_target->flags = HOST_DOWN;
+              current_target->reason.reason_id = ER_NOROUTE;
+            }
+          }
         }
       }
     }
   }
 
   /* Then we do the mass ping (if required - IP-level pings) */
-  if ((pingtype == PINGTYPE_NONE && !arpping_done) || hs->hostbatch[0]->ifType() == devt_loopback) {
+  if ((pingtype == PINGTYPE_NONE && !arpping_done) || current_target->ifType() == devt_loopback) {
     for (i=0; i < hs->current_batch_sz; i++) {
-      if (!(hs->hostbatch[i]->flags & HOST_DOWN || hs->hostbatch[i]->timedOut(&now))) {
-        initialize_timeout_info(&hs->hostbatch[i]->to);
-        hs->hostbatch[i]->flags |= HOST_UP; /*hostbatch[i].up = 1;*/
+      current_target = hs->hostbatch[i];
+      if (!(current_target->flags & HOST_DOWN || current_target->timedOut(&now))) {
+        initialize_timeout_info(&current_target->to);
+        current_target->flags |= HOST_UP; /*hostbatch[i].up = 1;*/
         if (pingtype == PINGTYPE_NONE && !arpping_done)
-          hs->hostbatch[i]->reason.reason_id = ER_USER;
+          current_target->reason.reason_id = ER_USER;
         else
-          hs->hostbatch[i]->reason.reason_id = ER_LOCALHOST;
+          current_target->reason.reason_id = ER_LOCALHOST;
       }
     }
   } else if (!arpping_done) {
